;TGA-ANALYSATOR BY NORMAN FESKE (NO/ESCAPE) IN 29.04.96

;TGA-header:
; offset | bytes | function
;----------------------------------------------------------------
; $00    |  1    | length of image-ID-field (0..no field)
; $01    |  1    | colormap type (0..no colormap)
; $02    |  1    | image type:
;        |       |   0..no picturedata in file
;        |       |   1..colormap picture, uncompressed
;        |       |   2..RGB picture, uncompressed
;        |       |   3..monochrome picture, uncompressed
;        |       |   9..colormap picture, RLE-compression
;        |       |  10..RGB picture, RLE-compression
;        |       |  11..monochrome picture, RLE-compression
;        |       |  32..Huffmann, Delta, RLE-compression
;        |       |  33..Huffmann, Delta, RLE-compression
; $03    |  2    |  colormap offset
; $05    |  2    |  colormap lenght
; $07    |  1    |  colormap entry size (16,24,32 bit)
; $08    |  2    |  koordinate nullpoint
; $0A    |  2    |  koordinate nullpoint
; $0C    |  2    |  width
; $0E    |  2    |  height
; $10    |  2    |  color resolution (1,8,24 bit)
; $11    |  1    |  image descriptor field
; $12    | <=256 |  image-ID-field
; ...    | <=1024|  colormap

export analysetga




TEXT


;ROUTINE TO CONVERT A TGA IMAGE TO 16BIT HICOLOR 320X240 FORMAT
analysetga:
	;a0:	source		(tga-data)s
	;a1:	destination (hi-color-screen)

	;push registers
	movem.l	d1-d7/a0-a6, -(sp)
	
	;clear screen
	move.l	#2*320*240/4-1, d0
	move.l	a1, a2
a_tga_clrscr:
	clr.l	(a2)+
	dbf		d0, a_tga_clrscr
	
	;image descriptor
	clr.l	d5
	move.b	11(a0), d5
	movea	d5, a4
	;a4:	image describtor

	;get picture size
	move.w	12(a0), d2
	rol.w	#8, d2
	cmpi.w	#320, d2
	bgt		a_tga_end
	;d2:	width
	move.w	14(a0), d3
	rol.w	#8, d3
	cmpi.w	#240, d3
	ble		a_tga_height_ok
	move.w	#240, d3
a_tga_height_ok:
	;d3:	height
	
	;get lineoffset
	move.l	#320, d7
	sub.w	d2, d7
	asl.l	d7
	;d7:	lineoffset
	
	;offset of picturedata
	clr.l	d6
	move.b  (a0), d6			;ID-field
	move.w	3(a0), d1			;palette
	rol.w	#8, d1
	;d1:	color map offset
	add.w	d1, d6
	add.w	#18, d6				;headeroffset
	;d6:	offset
		
	;image type = 24bit RGB image, uncompressed
	cmpi.b	#2, 2(a0)			;image type
	beq		a_tga_unco_rgb
	cmpi.b	#0, 2(a0)
	bne		no_unco24_rgb
a_tga_unco_rgb:
	cmp.b	#24, $10(a0)
	bne		no_unco24_rgb
	bsr		a24_tga	
no_unco24_rgb:

	;image type = 24bit RGB image, RLE-compressed
	cmp.b	#10, 2(a0)			;image type
	bne		no_co24_rgb
	cmp.b	#24, $10(a0)
	bne		no_co24_rgb
	bsr		a24_compr_tga
no_co24_rgb:

a_tga_end:

	move.l	a4, d0

	;pull registers
	movem.l	(sp)+, d1-d7/a0-a6

	;result: image descriptor field

rts




;ROUTINE TRANSFORMS UNCOMPRESSED 24BIT TGA DATA TO SCREEN
a24_tga:
	;d6:	headeroffset
	;d7:	lineoffset
	;a0:	TGA-data
	;a1:	screenadress
	
	;add.offset to picturedata
	adda.l	d6, a0

	;y-loop:
	subq.w	#1, d3
a24_tga_loopy:

	;x-loop:
	move.w	d2, d5
	subq.w	#1, d5
a24_tga_loopx:

	;read 24bit value with headeroffset
	move.l	(a0), d1
	addq.l	#3, a0

	;transform pixel
	bsr		a24_trans_pix

	;copy result to screen
	move.w	d4, (a1)+

	dbf		d5, a24_tga_loopx

	;add. lineoffset to screenadress
	adda.l	d7, a1

	dbf		d3, a24_tga_loopy
rts







;ROUTINE TRANSFORMS COMPRESSED 24BIT TGA DATA TO SCREEN
a24_compr_tga:
	;d2:	picture width
	;d3:	picture height
	;d6:	headeroffset
	;d7:	lineoffset
	;a0:	TGA-data
	;a1:	screenadress
	;a4:	image descriptor
	
	;Screenende berechnen
	move.l	a1, a2
	adda.l	#320*240*2, a2
	;a2:	Screenende
	
	;Screenanfang sichern
	move.l	a1, a3
	
	;add. headeroffset to TGA-data adress
	adda.l	d6, a0

	;y-loop: height as y-counter
	subq.l	#1, d3
	subq.l	#1, d2
a24c_tga_loopy:

	;d5:	x-loop-counter
	move.w	d2, d5

	;x-loop
a24c_tga_loopx:

	;headerbyte:
	clr.l	d0
	move.b	(a0)+, d0
	bclr	#7, d0						;next record compressed?
	;d0:	number of pixels in record
	beq		a24c_unco_rec

;compressed record:	
a24c_co_rec:

	;which pixelcolor?
	move.l	(a0), d1
	addq.l	#3, a0
	
	;transform the pixel
	bsr		a24_trans_pix
	;d4:	result
	
	;pixelloop: compressed data
	;d0:	loopcounter: numb.of compressed pixels in record
a24c_co_rloop:

	move.w	d4, (a1)+
	subq.w	#1, d5					;x-counter
	cmpi.w	#0, d5
	blt		a24c_next_line

	dbf		d0, a24c_co_rloop

	bra		a24c_end_rec
	
;uncompressed record:	
a24c_unco_rec:

	;pixelloop: raw-data
	;d0:	loopcounter: number of uncompressed pixels in record
a24c_unco_rloop:

	;read 24bit data
	move.l	(a0), d1
	addq.l	#3, a0
	
	;transform pixel
	bsr		a24_trans_pix
	;d4:	result
	
	;copy result to screen
	move.w	d4, (a1)+
	
	;x-counter
	subq.w	#1, d5
	cmpi.w	#0, d5
	blt		a24c_next_line	

	dbf		d0, a24c_unco_rloop

a24c_end_rec:

	bra		a24c_tga_loopx

a24c_next_line:

	;add. lineoffset to destination adress (screen)
	move.l	a1, d1
	add.l	d7, d1
	movea.l	d1, a1
	
	;test screenoverflow
	cmpa.l	a2, a1
	bgt		a24c_tga_end

	cmpa.l	a3, a1
	blt		a24c_tga_end

	dbf		d3, a24c_tga_loopy

a24c_tga_end:
rts










;ROUTINE TRANSFORMS ONE PIXEL
a24_trans_pix_old:
	;d1:	24-bit-value (long)
	;       rrrrrrrr gggggggg bbbbbbbb xxxxxxxx
	
	;generate 16-RGB-value:   rrrrrggggggbbbbb
	clr.w	d4
	
	;red
	btst.b	#7+8, d1
	beq		no_7_red
	bset	#15, d4
no_7_red:
	btst.b	#6+8, d1
	beq		no_6_red
	bset	#14, d4
no_6_red:
	btst.b	#5+8, d1
	beq		no_5_red
	bset	#13, d4
no_5_red:
	btst.b	#4+8, d1
	beq		no_4_red
	bset	#12, d4
no_4_red:
	btst.b	#3+8, d1
	beq		no_3_red
	bset	#11, d4
no_3_red:

	;green
	btst.b	#7+16, d1
	beq		no_7_green
	bset	#10, d4
no_7_green:
	btst.b	#6+16, d1
	beq		no_6_green
	bset	#9, d4
no_6_green:
	btst.b	#5+16, d1
	beq		no_5_green
	bset	#8, d4
no_5_green:
	btst.b	#4+16, d1
	beq		no_4_green
	bset	#7, d4
no_4_green:
	btst.b	#3+16, d1
	beq		no_3_green
	bset	#6, d4
no_3_green:
	btst.b	#2+16, d1
	beq		no_2_green
	bset	#5, d4
no_2_green:

	;blue
	btst.b	#7+24, d1
	beq		no_7_blue
	bset	#4, d4
no_7_blue:
	btst.b	#6+24, d1
	beq		no_6_blue
	bset	#3, d4
no_6_blue:
	btst.b	#5+24, d1
	beq		no_5_blue
	bset	#2, d4
no_5_blue:
	btst.b	#4+24, d1
	beq		no_4_blue
	bset	#1, d4
no_4_blue:
	btst.b	#3+24, d1
	beq		no_3_blue
	bset	#0, d4
no_3_blue:

	;d4:	result: 16bit RGB value
rts


;ROUTINE TRANSFORMS ONE PIXEL (OPTIMIZED VERSION)
a24_trans_pix:
	;d1:	24-bit-value (long)
	;       bbbbbbbb gggggggg rrrrrrrr xxxxxxxx

	move.w	d1, d4
	andi.w	#%1111100000000000, d4
	
	swap	d1
	;d1:	rrrrrrrr xxxxxxxx bbbbbbbb gggggggg
	
	ror.l	#8, d1
	ror.l	#3, d1
	;d1:    bbbggggg gggrrrrr rrrxxxxx xxxbbbbb
	
	andi.w	#%0000000000011111, d1
	add.w	d1, d4
	
	swap	d1
	;d1:    rrrxxxxx xxxbbbbb bbbggggg gggrrrrr
	
	lsr.w	#2, d1
	andi.w	#%0000011111100000, d1
	add.w	d1, d4	
	
	;d4:	result: 16 bit value
rts


END * OF * FILE